#!/usr/bin/env ruby
###########################################################################
###########################################################################
##  Copyright The Resistance (C) 1995-2011                               ##
##  Author: blass                                                        ##
##  blass[at]sdf.lonestar.org                                            ##
##  Badass 0.2 Beta                                                      ##
##  This file is part of Badass.                                         ##
##  Badass is free software: you can redistribute it and/or modify       ##
##  it under the terms of the GNU General Public License as published by ##
##  the Free Software Foundation, either version 3 of the License, or    ##
##  (at your option) any later version.                                  ##
##  Badass is distributed in the hope that it will be useful,            ##
##  but WITHOUT ANY WARRANTY; without even the implied warranty of       ##
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        ##
##  GNU General Public License for more details.                         ##
##  You should have received a copy of the GNU General Public License    ##
##  along with Badass. If not, see http://www.gnu.org/licenses/          ##
###########################################################################
###########################################################################

module Racket
  module L2
    # Miscelaneous L2 helper methods
    module Misc
      
      # given a string representing a MAC address, return the
      # human readable form
      def Misc.string2mac(string)
        string.unpack("C*").map { |i| i.to_s(16).ljust(2,"0") }.join(":")
      end
      
      # given a MAC address, return the string representation
      def Misc.mac2string(mac)
        mac.split(":").map { |i| i.hex.chr }.join
      end
      
      # given a MAC address, return the long representation
      def Misc.mac2long(addr)
        long = 0
        addr.split(':').map { |s| s.to_i(16) }.each do |o|
          long = (long << 8) ^ o
        end
        long
      end
      
      # given a long representing a MAC address
      # print it out in human readable form of a given length, 
      # defaulting to 6 (ethernet)
      def Misc.long2mac(long, len=6)
        long.to_s(16).rjust(len*2, '0').unpack("a2"*len).join(":")
      end
      
      # Return a random MAC, defaults to 6 bytes (ethernet)
      def Misc.randommac(len=6)
        long2mac(rand(2**(8*len)), len)
      end
    end
  end
end
# vim: set ts=2 et sw=2:
