#!/usr/bin/env ruby
###########################################################################
###########################################################################
##  Copyright The Resistance (C) 1995-2011                               ##
##  Author: blass                                                        ##
##  blass[at]sdf.lonestar.org                                            ##
##  Badass 0.2 Beta                                                      ##
##  This file is part of Badass.                                         ##
##  Badass is free software: you can redistribute it and/or modify       ##
##  it under the terms of the GNU General Public License as published by ##
##  the Free Software Foundation, either version 3 of the License, or    ##
##  (at your option) any later version.                                  ##
##  Badass is distributed in the hope that it will be useful,            ##
##  but WITHOUT ANY WARRANTY; without even the implied warranty of       ##
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        ##
##  GNU General Public License for more details.                         ##
##  You should have received a copy of the GNU General Public License    ##
##  along with Badass. If not, see http://www.gnu.org/licenses/          ##
###########################################################################
###########################################################################

module Racket
  module Misc
    # Simple class for your average type, length, value datastructure.
    # Everything after the TLV is stuff into +rest+
    class TLV
      attr_accessor :type, :length, :value, :rest, :lbytes, :tlinclude
      
      # Create a new TLV which requires +ts+ bytes for the type field
      # and +ls+ bytes for the length field, where (optionally) the value
      # in +length+ is a multiple of +lbytes+ and (optionally) whether or not
      # the length field indicates the total length of the TLV of just that of 
      # the value
      def initialize(ts, ls, lbytes=1, tlinclude=false)
        @ts = ts
        @ls = ls
        @lbytes = lbytes
        @tlinclude = tlinclude
      end 
      
      # Given +data+, return the type, length, value and rest 
      # values as dictated by this instance.
      def decode(data)
        s = "#{punpack_string(@ts)}#{punpack_string(@ls)}"
        type, length, tmp = data.unpack("#{s}a*")
        if (type.nil? or length.nil?)
          nil
        else
          elength = (length * lbytes) - (@tlinclude ? (@ls + @ts) : 0) 
          value, rest = tmp.unpack("a#{elength}a*")
          if (value.empty? and length > 0)
            nil
          else
            [type, length, value, rest]
          end
        end
      end
      
      def decode!(data)
        @type, @length, @value, @rest = self.decode(data)
      end
      
      # Return a string suitable for use elswhere.
      def encode
        s = "#{punpack_string(@ts)}#{punpack_string(@ls)}"
        [@type, @length, @value].pack("#{s}a*")
      end
      
      def to_s
        encode
      end
      
      def to_str
        encode
      end
      
      private
      # XXX: make this handle arbitrarily sized fields
      def punpack_string(size)
        s = ""
        case size
        when 1
          s << "C"
        when 2
          s << "n"
        when 4
          s << "N"
        else
          raise ArgumentError, "Size #{s} is not a supported conversion size"
        end
        s
      end
      
    end
  end
end
# vim: set ts=2 et sw=2:
