#!/usr/bin/env ruby
###########################################################################
###########################################################################
##  Copyright The Resistance (C) 1995-2011                               ##
##  Author: blass                                                        ##
##  blass[at]sdf.lonestar.org                                            ##
##  Badass 0.2 Beta                                                      ##
##  This file is part of Badass.                                         ##
##  Badass is free software: you can redistribute it and/or modify       ##
##  it under the terms of the GNU General Public License as published by ##
##  the Free Software Foundation, either version 3 of the License, or    ##
##  (at your option) any later version.                                  ##
##  Badass is distributed in the hope that it will be useful,            ##
##  but WITHOUT ANY WARRANTY; without even the implied warranty of       ##
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        ##
##  GNU General Public License for more details.                         ##
##  You should have received a copy of the GNU General Public License    ##
##  along with Badass. If not, see http://www.gnu.org/licenses/          ##
###########################################################################
###########################################################################

require 'socket'
require File.dirname(__FILE__) + '/racketpart.rb'
require File.dirname(__FILE__) + '/misc.rb'
require File.dirname(__FILE__) + '/l2.rb'
require File.dirname(__FILE__) + '/l3.rb'
require File.dirname(__FILE__) + '/l4.rb'
require File.dirname(__FILE__) + '/l5.rb'

module Racket
  class Racket
    
    attr_accessor :iface, :mtu, :timeout
    attr_accessor :layers, :payload
    
    @@loaded_pcaprub = false
    begin
      require 'pcaprub'
      @@loaded_pcaprub = true
    rescue ::LoadError
    end
    
    
    def initialize(payload="")
      @layers = []
      @mtu = 1500
      @timeout = 10
      @payload = payload
      1.upto(7) do |l|
        self.class.send(:define_method, "layer#{l}", lambda { @layers[l] })
        self.class.send(:define_method, "l#{l}", lambda { @layers[l] })
        self.class.send(:define_method, "layer#{l}=", lambda { |x| @layers[l] = x; })
        self.class.send(:define_method, "l#{l}=", lambda { |x| @layers[l] = x; })
      end
    end
    
    # Assemble all the pieces of this Racket as a string, ready for sending.
    def pack
      last_payload = ""
      orig_payload = ""
      @layers.compact.reverse.each do |l|
        # save the original payload
        orig_payload = l.payload
        # tack on the last payload in
        # case fix needs it...
        l.payload += last_payload
        if (l.autofix?)
          l.fix!
        end
        
        if (l.payload == orig_payload + last_payload)
          # payload was not modified by fix, so reset it to what
          # it used to be
          l.payload = orig_payload
        else
          # payload was modified by fix.  chop off what we added.
          # XXX: this assumes that what we added is still at the end.  
          # XXX: this is not always true
          l.payload = l.payload.slice(0, l.payload.length - last_payload.length)
        end
        
        # save this layer for the next guy
        last_payload += l
        
      end
      
      payload = ""
      @layers.compact.each do |l|
        payload += l
      end
      payload
    end
    
    # return a pretty interpretation of this packet
    def pretty
      s = ""
      @layers.compact.each do |l|
        s << "#{l.class}: "
        s << l.pretty
        s << "\n"
      end
      s
    end
    
    
    # Attempt to figure out which of send2 or send3 needs to be called.
    def sendpacket
      if (@layers[2])
        send2
      else
        send3
      end
    end
    
    # Write raw layer2 frames
    def send2
      if(not @@loaded_pcaprub)
        raise RuntimeError, "Could not initialize the pcaprub library (You need pcaprub from SVN (http://rubyforge.org/projects/pcaprub/))" 
      end
      begin
        
        p = Pcap::open_live(@iface, @mtu, false, @timeout)
      rescue Exception => e
        puts "Pcap: can't open device '#{@iface}' (#{e})"
        return
      end
      
      begin
        b = p.inject(pack)
        #p.pcap_close
        return b
      rescue Exception => e
        puts "Pcap: error while sending packet on '#{@iface}' (#{e})"
      end
    end
    
    # Write raw layer3 frames
    def send3
      begin
        s = Socket.open(Socket::PF_INET, Socket::SOCK_RAW, Socket::IPPROTO_RAW)
        
        if (Socket.const_defined?('SOL_IP'))
          s.setsockopt(Socket::SOL_IP, Socket::IP_HDRINCL, true)
        else
          # BSD
          s.setsockopt(Socket::IPPROTO_IP, Socket::IP_HDRINCL, true)
        end
      rescue Errno::EPERM
        raise ArgumentError, "Must run #{$0} as root."
      end
      
      return s.send(pack, 0, Socket.pack_sockaddr_in(1024, @layers[3].dst_ip))
    end
  end
end

# vim: set ts=2 et sw=2:
